import express, { Request, Response } from 'express';
import fs from 'fs';
import path from 'path';
import axios from 'axios';

const app = express();
const PORT = process.env.EMS_PORT;

const tileFolder = path.join(__dirname, '..', 'tiles');

const fetchTileFromAPI = async (provider: string, z: string, x: string, y: string): Promise<Buffer> => {
    const url = provider === 'google'
        ? `https://khms2.google.com/kh/v=984?x=${x}&y=${y}&z=${z}`
        : `https://core-sat.maps.yandex.net/tiles?l=sat&x=${x}&y=${y}&z=${z}&scale=1&lang=ru_RU`;

    const response = await axios.get(url, { responseType: 'arraybuffer' });
    return response.data;
};

app.get('/tile/:provider/:z/:x/:y', async (req: Request, res: Response) => {
    const { provider, z, x, y } = req.params;

    if (!['google', 'yandex'].includes(provider)) {
        return res.status(400).send('Invalid provider');
    }

    const tilePath = path.join(tileFolder, provider, z, x, `${y}.jpg`);

    if (fs.existsSync(tilePath)) {
        return res.sendFile(tilePath);
    }

    try {
        const tileData = await fetchTileFromAPI(provider, z, x, y);

        fs.mkdirSync(path.dirname(tilePath), { recursive: true });

        fs.writeFileSync(tilePath, tileData);

        res.contentType('image/jpeg');
        res.send(tileData);
    } catch (error) {
        console.error('Error fetching tile from API:', error);
        res.status(500).send('Error fetching tile from API');
    }
});

app.listen(PORT, () => console.log(`Server running on port ${PORT}`));
